@@ -29,11 +29,11 @@ public:
 
 KDateValidator::KDateValidator(QObject *parent)
     : QValidator(parent)
-    , d(nullptr)
 {
-    Q_UNUSED(d);
 }
 
+KDateValidator::~KDateValidator() = default;
+
 QValidator::State KDateValidator::validate(QString &text, int &unused) const
 {
     Q_UNUSED(unused);
